@@ -1,5 +1,5 @@
 use derive_more::Display;
-use poem::{get, handler, http::StatusCode, route, Response, ResponseError, Server};
+use poem::{get, http::StatusCode, route, Response, ResponseError, Server};
 
 #[derive(Debug, Display)]
 struct CustomError;
@@ -12,14 +12,14 @@ impl ResponseError for CustomError {
     }
 }
 
-#[handler]
+#[get]
 fn index() -> Result<String, CustomError> {
     Err(CustomError)
 }
 
 #[tokio::main]
 async fn main() {
-    let app = route().at("/", get(index));
+    let app = route().at("/", index);
     let server = Server::bind("127.0.0.1:3000").await.unwrap();
     server.run(app).await.unwrap();
 }
