@@ -1,6 +1,6 @@
 use std::future::Future;
 
-use crate::{Endpoint, Request, Response, Result};
+use crate::{Endpoint, IntoResponse, Request, Response, Result};
 
 /// Endpoint for the [`map`](super::EndpointExt::map) method.
 pub struct Map<E, F> {
@@ -16,13 +16,16 @@ impl<E, F> Map<E, F> {
 }
 
 #[async_trait::async_trait]
-impl<E, F, Fut> Endpoint for Map<E, F>
+impl<E, F, Fut, R> Endpoint for Map<E, F>
 where
     E: Endpoint,
     F: Fn(Result<Response>) -> Fut + Send + Sync + 'static,
-    Fut: Future<Output = Result<Response>> + Send + 'static,
+    Fut: Future<Output = Result<R>> + Send + 'static,
+    R: IntoResponse,
 {
     async fn call(&self, req: Request) -> Result<Response> {
-        (self.f)(self.inner.call(req).await).await
+        (self.f)(self.inner.call(req).await)
+            .await
+            .and_then(IntoResponse::into_response)
     }
 }
