@@ -3,16 +3,20 @@
 mod and_then;
 #[allow(clippy::module_inception)]
 mod endpoint;
+mod guard_endpoint;
 mod map;
 mod map_err;
 mod map_ok;
 mod map_request;
 mod map_to_response;
+mod or;
 
 pub use and_then::AndThen;
 pub use endpoint::{Endpoint, EndpointExt};
+pub use guard_endpoint::GuardEndpoint;
 pub use map::Map;
 pub use map_err::MapErr;
 pub use map_ok::MapOk;
 pub use map_request::MapRequest;
 pub use map_to_response::MapToResponse;
+pub use or::Or;
