@@ -44,3 +44,20 @@ where
         self.inner.call(req).await
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::{handler, EndpointExt};
+
+    #[tokio::test]
+    async fn test_add_data() {
+        #[handler(internal)]
+        async fn index(req: &Request) {
+            assert_eq!(req.extensions().get::<i32>(), Some(&100));
+        }
+
+        let app = index.with(AddData::new(100i32));
+        app.call(Request::default()).await.unwrap();
+    }
+}
