@@ -107,8 +107,8 @@ impl Response {
     }
 
     /// Sets the body for this response.
-    pub fn set_body(&mut self, body: Body) {
-        self.body = body;
+    pub fn set_body(&mut self, body: impl Into<Body>) {
+        self.body = body.into();
     }
 
     /// Take the body from this response and sets the body to empty.
